function [G] = E04_AM_ratio(r,gamma,series_length)
%E04_AM_ratio - power series solution for outer wind profile 
% as function of radius - this function computes the ratio of the
% azimuthal wind to the wind that would be obtained from angular momentum
% conservation of inflow from rest at radius r_0
% 
% input arguments:
% r = nondimensional array of r/r_0 - k elements in length
% gamma = cD*f*r_0/w_r = r_0/(w_Q Ro) [dimensionless]
%       key solution control parameter, typically >>1, 
%       equal to r_0/w_Q from Emanuel 2004 and gamma from Chavas & Lin, 2016 
%  *** THIS FUNCTION ASSUMES gamma IS A SCALAR -- use E04_AM_ratio_matrix
%  for multiple values of gamma at once ***
% series_length = N = number of terms to calculate in power series 
%
% output arguments:
% G = V/V_AMC = ratio of azimuthal wind speed to wind speed of
% angular-momentum-conserving profile
%
% Author: Tim Cronin, MIT; twcronin@mit.edu

%------------- BEGIN CODE --------------

% coefficients in series for denominator and numerator, respectively
an_d = zeros(series_length,1);
an_n = zeros(series_length,1);

an_d(1) = 1;      % x^0 term  
if series_length>=2
    an_d(2) = gamma;    % x^1 term
    an_n(1) = 1/gamma*1*an_d(2);
    if series_length>=3
        an_d(3) = gamma^2/4; % x^2 term
        an_n(2) = 1/gamma*2*an_d(3);
        if series_length>=4
            for i=3:series_length-1        
                % recurrence relation: a_n = [1/(2n^2)]([2 gamma + n(n-1)] a_{n-1) - [2 gamma] a_{n-2})
                % n = i+1 for array indices, but n=i when used algebraically below
                an_d(i+1) = (1/(2*i^2))*(an_d(i)*(2*gamma+i*(i-1)) - an_d(i-1)*2*gamma);
                an_n(i) = 1/gamma*i*an_d(i+1);
            end
        end
    end
end

% RN: matrix of [(1-r)^0; (1-r)^1; (1-r)^2; ...; (1-r)^(series_length-1)]
RN = ones(length(r),series_length);
for i=1:series_length-1
    RN(:,i+1) = RN(:,i).*(1-r)';
end
num = RN*an_n;
den = RN*an_d;

G = num./den;

end

