function [G] = E04_AM_ratio_matrix(r,gamma,series_length)
%E04_AM_ratio - power series solution for outer wind profile 
% as function of radius - this function computes the ratio of the
% azimuthal wind to the wind that would be obtained from angular momentum
% conservation of inflow from rest at radius r_0
% 
%
% input arguments:
% r = nondimensional array of r/r_0 - K elements in length
% gamma = cD*f*r_0/w_r = r_0/(w_Q Ro) [dimensionless]
%       key solution control parameter, typically >>1, 
%       equal to r_0/w_Q from Emanuel 2004 and gamma from Chavas & Lin, 2016 
% gamma can be an array of multiple different values - L elements in length
% series_length = N = number of terms to calculate in power series 
%
% output arguments:
% G = V/V_AMC = ratio of azimuthal wind speed to wind speed of
% angular-momentum-conserving profile
%
% solution method is to construct two NxN matrices of the coefficients that
% left-multiply powers of Gamma (a NxK matrix) to give the numerator and 
% denominator power series coefficients (N terms for each of the K elements of Gamma), 
% then a matrix of powers of x=(1-r/r0) (size LxN) left-multiplies this
% result
%
% Author: Tim Cronin, MIT; twcronin@mit.edu

%------------- BEGIN CODE --------------

N = series_length;
K = length(r);
if ~iscolumn(r)
    r = r';
end
L = length(gamma);
if ~isrow(gamma)
    gamma = gamma';
end

% coefficients in series for denominator and numerator, respectively -
% initialize with identity matrix
PP_den = eye(N+1,N+1);
PP_num = eye(N,N);

% construct matrix of powers of (1-r) (nth column is (1-r)^(n-1)
XX = ones(K,N);
for i=2:N
    XX(:,i) = XX(:,i-1).*(1-r);
end

% construct matrix of powers of gamma (nth row is gamma.^(n-1))
GAMMA = ones(N,L);
for i=2:N
    GAMMA(i,:) = GAMMA(i-1,:).*gamma;
end

% construct coefficient matrix (NxN), nth row is for x.^(n-1) and mth column is for gamma.^(n-1)
% diagonal follows PP(i,i) = 1./((i-1)!)^2, or recurrence that each diagonal element is only related to the previous one
% PP(4,3) has value -1/36
% PP(i>1,1) and PP(i>2,2) all = 0
% PP(:,3) for i>4 is equal to PP(i-1,3)*(i-2)/(2*(i-1)) -- special case of 
% general lower-diagonal entries with j>=3 and i>j which has recurrence given by:
% PP(i,j) = PP(i-1,j-1)/(i-1)^2 + PP(i-1,j)*(i-2)/(2*(i-1)) - PP(i-2,j-1)/(i-1)^2  
PP_den(4,3) = -1/36;
for j=3:N+1 % loop over columns (index j)
    for i=3:N+1 % loop over rows (index i)
        if j==i
            PP_den(i,j) = PP_den(i-1,j-1)/(i-1)^2;
        elseif (j>=3)&&(i>j)&&(i>4)
            PP_den(i,j) = PP_den(i-1,j-1)/(i-1)^2 + PP_den(i-1,j)*(i-2)/(2*(i-1)) - PP_den(i-2,j-1)/(i-1)^2;
        end
    end
end
for i=2:N % fill numerator matrix
    PP_num(i,:) = i*PP_den(i+1,2:N+1);
end

num = XX*(PP_num*GAMMA);
den = XX*(PP_den(1:N,1:N)*GAMMA); % truncate denominator coefficients to NxN

G = num./den;

end

