%% Fig01_TC_wind_schematic
% a) plot V(r) for inner, outer, and AMC profiles, 
% with labels for V_m, r_m, r_0
% b) plot \psi(r) with cartoon of vertical motion and clouds
%
%
% Script for Figure 1 of Cronin, 2023, GRL - "An analytic model for
% Tropical cyclone outer winds"
% 
% Author: Tim Cronin, MIT; twcronin@mit.edu
%
% Functions used: 
%   TC23_windprofile
%   E04_AM_ratio
%   gcfsavepdf

%------------- BEGIN CODE --------------

% dimensional variables for example TC
vm      = 50;    % m/s
f       = 5e-5;  % 1/s
rm      = 50000; % m
wr      = 0.002; % m/s
cD      = 0.001; % -

Ro = vm/(f*rm);
wQ = wr/(cD*vm);

[r,V,r0,rmerge] = TC23_windprofile(vm,rm,f,cD,wr);

V_AMC = f*(r0^2-r.^2)./(2*r);
G_out = E04_AM_ratio(r/r0,cD*f*r0/wr,100);
V_out = V_AMC'.*G_out;
nitvin = 10;
Ro_in = Ro; % inner wind profile 'Ro' will not exactly equal prescribed Ro = vm/(f rm)
for k=1:nitvin
    rfac = sqrt(sqrt(4*Ro_in.^2 + 12*Ro_in + 5) - 2*Ro_in - 2); % factor relating rm and rm'
    vfac = 2.*Ro_in*(1+rfac^2)/(rfac*((4*Ro_in+1)-rfac^2)); % factor relating vm and vm'
    rm_in = 1./rfac; % tweak the value of rm' in ER11 model
    vm_in = 1*vfac; % tweak the value of rm' in ER11 model
    Ro_in = Ro*vm_in/rm_in; % tweak the value of Ro' and repeat a few times
end
rm_in = rm*rm_in;
r0_in = sqrt(4*Ro_in+1)*rm_in;
r_in = (0:0.001:1)*r0_in;
V_in = (f*(r0_in.^2-r_in.^2)./(2*r_in)).*((r_in/rm_in).^2./(1+(r_in/rm_in).^2));

psi_out = 0.5*wr*(r0.^2-r.^2);
psi_in  = cD*vm*rm^2/(16*Ro_in^2)*(1+1/(2*Ro_in))*(r/rm).^3.*((r0_in/rm_in).^2-(r/rm_in).^2).^2;
psi_in = psi_in.*(r<r0_in);

ra_rm = rmerge/rm;
psi_merged = psi_out.*((r/rm)>ra_rm)+psi_in.*((r/rm)<=ra_rm);

%% make figures
figure('Position',[50 50 600 600]);

subplot(2,1,1);
hold on; 
set(gca,'Fontsize',16)
plot(r/1000, V, 'k-', 'Linewidth',3);
plot(r/1000, V_out, 'c--','Linewidth',1.5)
plot(r_in/1000, V_in, 'r--','Linewidth',1.5)
plot(r/1000,V_AMC, 'g-', 'Linewidth',1);
ylim([0 1.5*vm])


box on;
set(gca,'XTickLabel',{},'YtickLabel',{})
line(1/1000*[0 rm],[vm vm], 'Color',[0.5 0.5 0.5],'Linestyle','-')
line(1/1000*[rm rm],[0 vm], 'Color',[0.5 0.5 0.5],'Linestyle','-')
line(ra_rm/1000*[rm rm],[0 1.5*vm], 'Color',[0.5 0.5 0.5],'Linestyle','-')
line(1/1000*[r0 r0],[0 1.5*vm], 'Color',[0.5 0.5 0.5],'Linestyle','-')
text(rm/1000,-vm/10, 'r_m', 'HorizontalAlignment','Center','Fontsize',16,'Color',[0.5 0.5 0.5])
text(ra_rm*rm/1000,-vm/10, 'r_a', 'HorizontalAlignment','Center','Fontsize',16,'Color',[0.5 0.5 0.5])
text(-1/1000*rm,vm, 'V_m', 'VerticalAlignment','Middle','Fontsize',16,'Color',[0.5 0.5 0.5])
text(r0/1000,-vm/10, 'r_0', 'HorizontalAlignment','Center','Fontsize',16,'Color',[0.5 0.5 0.5])
legend('Merged wind profile', 'Outer wind profile', 'Inner wind profile', 'AMC limit')

ylabel({'Azimuthal wind, V'; ' '})
xlabel('Radius, r')
title('a) Azimuthal wind profile', 'Fontweight', 'normal');

subplot(2,1,2);
hold on; 
set(gca,'Fontsize',16)
plot(r/1000, psi_merged, 'k-', 'Linewidth',3);
plot(r/1000, psi_out, 'c--','Linewidth',0.75)
plot(r/1000, psi_in, 'r--','Linewidth',0.75)
line(ra_rm/1000*[rm rm],[0 3*max(psi_out)], 'Color',[0.5 0.5 0.5],'Linestyle','-')
text(ra_rm*rm/1000,-max(psi_out)/5, 'r_a', 'HorizontalAlignment','Center','Fontsize',16,'Color',[0.5 0.5 0.5])

box on;
set(gca,'XTickLabel',{},'YtickLabel',{})
ylim([0 3*max(psi_out)])

% draw a cloud inside r_a
cld_ang = (0:0.01:2*pi);
cld_rfac = zeros(length(cld_ang),1);
for i=3:length(cld_ang)-1
    cld_rfac(i+1) = 0.95*cld_rfac(i)+0.02*randn(1)-0.2*cld_rfac(i-1)+0.2*cld_rfac(i-2);
end
cld_rfac = cld_rfac+0.9;
patch(1.5*rm/1000+rm/1000*cos(cld_ang).*cld_rfac', ...
     2*max(psi_out)+max(psi_out)*sin(cld_ang).*cld_rfac', [0.8 0.8 0.8],'EdgeColor','w')    


% draw arrows for circulation
annotation('arrow',[0.15 0.15],[0.24 0.27],'Color',[0 0 1])
annotation('arrow',[0.16 0.16],[0.24 0.28],'Color',[0 0 1])
annotation('arrow',[0.17 0.17],[0.24 0.29],'Color',[0 0 1])
annotation('arrow',[0.18 0.18],[0.24 0.30],'Color',[0 0 1])
annotation('arrow',[0.19 0.19],[0.24 0.31],'Color',[0 0 1])
annotation('arrow',[0.20 0.20],[0.24 0.32],'Color',[0 0 1])
for i=1:20
    annotation('arrow',[0.21+0.035*(i-1) 0.21+0.035*(i-1)],[0.24 0.22],'Color',[0 0 1])
end

xlabel('Radius, r')
ylabel({'Overturning';'Circulation, \psi'})
text(1/1000*8*rm, 1.25*max(psi_out), '\color{blue} Radiative subsidence w_r','FontSize',16)

legend('Merged overturning streamfunction','Outer streamfunction','Inner streamfunction')

title('b) Overturning circulation', 'Fontweight', 'normal');

gcfsavepdf('/Users/timothycronin/Dropbox/ConfsTalksPubs/Cronin2023_TC-OuterWindModel/Fig01_TC_wind_schematic.pdf')

