%% Fig03_rar0_vs_RowQ_mod.m
% Script to loop over different inner wind profiles and find the matching 
% outer wind profile using the new expression for the outer wind profile 
%
%
% Script for Figure 3 of Cronin, 2023, GRL - "An analytic model for
% Tropical cyclone outer winds"
% 
% This script is also used to make and save the lookup tables for r_0 and
% r_a that are accessed by the function TC23_windprofile. 
% Default lookup table bounds and resolution are coded into the definitions 
% of w_Q and Ro below - modify these to make a different table   
%
% Author: Tim Cronin, MIT; twcronin@mit.edu
%
% Functions used: 
%   E04_AM_ratio
%   gcfsavepdf

%------------- BEGIN CODE --------------

% variable      value           units               description
% -------------------------------------------------------------------------
r  =            (0:0.0001:1);   % -                 radius of r/r_0
%Ro =            (5:5:50);       % -                 inner core Rossby #, vm/(f rm)
%Ro =            logspace(0,2,30); % -                 inner core Rossby #, vm/(f rm)
%wQ =            logspace(-2,1,40); % -              nondimensional radiative subsidence velocity, -w_r/(cd vm)
Ro =            logspace(0,2,90); % -                 inner core Rossby #, vm/(f rm)
wQ =            logspace(-2,1,120); % -              nondimensional radiative subsidence velocity, -w_r/(cd vm)
%Ro = 5;
%wQ = 0.1;
tol=            0.0001;         % -                 relative error tolerance for wind match at r=r_a
nit_max =       40;             % -                 maximum number of permitted iterations

r0 = zeros(length(Ro),length(wQ)); % output array of outer radii relative to rm
ra = zeros(length(Ro),length(wQ)); % output array of merge radii relative to rm
GAMMA = zeros(length(Ro),length(wQ)); % output array of gamma values
r0_approx = zeros(length(Ro),length(wQ));
G_ra = zeros(length(Ro),length(wQ)); % ratio of wind to AMC wind at merge radius
r0_inner = zeros(length(Ro),length(wQ));
nit = zeros(length(Ro),length(wQ));
r0_guesses = NaN(nit_max,1);
errs = NaN(nit_max,1);

tic;

for i=1:length(Ro)
    for j=1:length(wQ)
        % initialize guess and error arrays with two guesses: one at 
        % r0_guess(1) = r0_inner, and the other using a 'smart' slightly larger value of
        % r0_guess(2) = r0_inner*sqrt(1+1/(4*wQ))
        % then start iteration 
        r0_guesses = NaN(nit_max,1);
        errs = NaN(nit_max,1);
        
        % tweak inner wind model slightly to make sure that prescribed 
        % vm and rm actually wind up as max(v) and radius of max(v)
        % here, vm and rm are dimensionless and each = 1, so want to
        % compute vm_in, rm_in, and Ro_in that are consistent with this
        % choice
        nitvin = 10;
        Ro_in = Ro(i); % inner wind profile 'Ro' will not exactly equal prescribed Ro = vm/(f rm)
        for k=1:nitvin
            rfac = sqrt(sqrt(4*Ro_in.^2 + 12*Ro_in + 5) - 2*Ro_in - 2); % factor relating rm and rm'
            vfac = 2.*Ro_in*(1+rfac^2)/(rfac*((4*Ro_in+1)-rfac^2)); % factor relating vm and vm'
            rm_in = 1./rfac; % tweak the value of rm' in ER11 model
            vm_in = 1*vfac; % tweak the value of rm' in ER11 model
            Ro_in = Ro(i)*vm_in/rm_in; % tweak the value of Ro' and repeat a few times
        end
        ri = (0:0.0001:1)* sqrt(4*Ro_in+1)/rfac; % array of inner wind radii
        V_inner = vfac*(rfac*ri)./(2*Ro_in.*(1+(rfac*ri).^2)).*((4*Ro_in+1)-(rfac*ri).^2);
        r0_inner(i,j) = sqrt(4*Ro_in+1)/rfac;
        
        % guess 1 at r0_inner
        nit(i,j) = nit(i,j)+1;
        r0_guess = r0_inner(i,j);
        r0_guesses(nit(i,j)) = r0_guess;
        gamma = r0_guess./(wQ(j).*Ro(i));
        nterms = floor(50+2*sqrt(gamma));
        G = E04_AM_ratio(r,gamma,nterms);
        V_out = r0_guess.*(1-r.^2)./(2.*Ro(i).*r).*G';
        V_in1 = interp1(ri,V_inner,r0_guess*r);
        V_in1 = V_in1.*(r0_guess*r<=sqrt(r0_inner(i,j)));    
        diff = (V_out - V_in1);
        err = abs(min(diff));
        errs(nit(i,j)) = min(diff);
     
        if err<tol 
            % if there is no outer wind portion of the solution,
            % first guess will be right and can avoid further iterations 
            r0(i,j) = r0_guess;
            ra(i,j) = r0_guess*r(diff==min(diff));
            G_ra(i,j) = G(diff==min(diff));
        else
            % guess 2 outer wind r0 scales with inner wind radius
            % and a function of wQ (based on emprical findings)
            nit(i,j) = nit(i,j)+1;
            r0_guess = r0_inner(i,j).*sqrt(1+1/wQ(j));
            r0_guesses(nit(i,j)) = r0_guess;
            gamma = r0_guess./(wQ(j).*Ro(i));
            nterms = floor(50+2*sqrt(gamma));
            G = E04_AM_ratio(r,gamma,nterms);
            V_out = r0_guess.*(1-r.^2)./(2.*Ro(i).*r).*G';
            V_in1 = interp1(ri,V_inner,r0_guess*r);
            V_in1 = V_in1.*(r0_guess*r<=sqrt(r0_inner(i,j)));    
            diff = (V_out - V_in1);
            err = abs(min(diff));
            errs(nit(i,j)) = min(diff);
            
            while err>tol
            % now use r0_guesses and errs arrays to iterate forward
                nit(i,j) = nit(i,j)+1;
                r0_guess = interp1(errs(~isnan(errs)), r0_guesses(~isnan(errs)), 0, 'spline');
                r0_guesses(nit(i,j)) = r0_guess;
                gamma = r0_guess./(wQ(j).*Ro(i));
                nterms = floor(50+2*sqrt(gamma));
                G = E04_AM_ratio(r,gamma,nterms);
                V_out = r0_guess.*(1-r.^2)./(2.*Ro(i).*r).*G';
                V_in1 = interp1(ri,V_inner,r0_guess*r);
                V_in1 = V_in1.*(r0_guess*r<=sqrt(r0_inner(i,j)));    
                diff = (V_out - V_in1);
                err = abs(min(diff));
                errs(nit(i,j)) = min(diff);
            end
            
            r0(i,j) = r0_guess;
            ra(i,j) = r0_guess*r(diff==min(diff));
            G_ra(i,j) = G(diff==min(diff));
        end % iteration if-else-while loop
        GAMMA(i,j) = r0(i,j)./(Ro(i).*wQ(j));
        r0_approx(i,j) = sqrt(4*Ro(i)+1)*(sqrt(1+1/(4*wQ(j)))); 
    end
end

toc;

%% make plot of r_0 and r_a against Ro and wQ

% include points from Chavas, Lin, and Emanuel, 2015, Figure 10
% for representative examples/median storms in 6 intensity bins: 
%       TS-lo, TS-hi, Cat1, Cat2, Cat3, Cat4/5
CLE_vm = [17.0, 21.7, 27.5, 34.0, 39.5, 47.5]; % units m/s
CLE_rm = [60.1, 53.9, 38.8, 38.3, 24.2, 23.8]; % units **km**
CLE_f  = [ 5.5,  6.1,  6.0,  6.1,  5.6,  5.1]*1e-5; % units 1/s

CLE_Cd_approx = 1e-3; % my choice of Cd, not varying by category, to calulate wQ
CLE_wcool = 2e-3; % value of radiative-subsidence rate in CLE15

CLE_Ro = CLE_vm./(1000*CLE_f.*CLE_rm);
CLE_wQ = CLE_wcool./(CLE_vm.*CLE_Cd_approx);

CLE_colors = [0.7 0.7 0.7;
              0.4 0.4 0.4;
              0.1 0.8 0.1;
              0.8 0.8 0.1;
              1.0 0.4 0.1;
              1.0 0.0 0.0];

figure('Position',[50 50 350 1000]);

% part of plots will be grayed out because there is basically no merge:
% it's just all inner wind profile

ha1 = subplot(3,1,1);
hold on;
% part of plots will be grayed out because there is basically no merge:
% it's just all inner wind profile. plot this first
[C0,h0] = contour(wQ,Ro,ra./r0,[0.95 100],'k--');
Cgray = [C0(1,2:end), max(wQ), max(wQ); C0(2,2:end), min(Ro), max(Ro)];
patch(Cgray(1,:),Cgray(2,:),[0.5 0.5 0.5])
[C1,h1] = contour(wQ,Ro,r0,[5 10 15 20 30 40 50 60]);
%[C1a,h1a] = contour(wQ,Ro,r0_approx,[5 10 15 20 30 40 50 60],'LineStyle','--');
% add scaling for wQ* (no outer-wind solution for wQ>wQ*)
plot(16*sqrt(Ro)/27,Ro,':','Color',[0.25 0.25 0.25],'LineWidth',2) 
% add representative real storms from CLE15
scatter(CLE_wQ, CLE_Ro, 16, CLE_colors, 'filled')
set(gca,'Fontsize',14)
%xlabel('w_Q = -w_r/(c_D v_m)')
ylabel('Ro = $V_m/(f r_m)$','Interpreter','latex')
set(gca,'Xscale','log')
set(gca,'Yscale','log')
box on;
clabel(C1,h1)
title('a) $r_0/r_m$ against $\tilde{w_Q}$ and Ro','Fontweight','normal','Interpreter','latex')

ha2 = subplot(3,1,2);
hold on;
patch(Cgray(1,:),Cgray(2,:),[0.5 0.5 0.5])
[C2,h2] = contour(wQ,Ro,ra,(1.5:0.5:5));
% add scaling for wQ* (no outer-wind solution for wQ>wQ*)
plot(16*sqrt(Ro)/27,Ro,':','Color',[0.25 0.25 0.25],'LineWidth',2) 
% add representative real storms from CLE15
scatter(CLE_wQ, CLE_Ro, 16, CLE_colors, 'filled')
set(gca,'Fontsize',14)
%xlabel('w_Q = -w_r/(c_D v_m)')
ylabel('Ro = $V_m/(f r_m)$','Interpreter','latex')
set(gca,'Xscale','log')
set(gca,'Yscale','log')
box on;
clabel(C2,h2)
title('b) $r_a/r_m$ against $\tilde{w_Q}$ and Ro','Fontweight','normal','Interpreter','latex')

ha3 = subplot(3,1,3);
hold on;
patch(Cgray(1,:),Cgray(2,:),[0.5 0.5 0.5])
[C3,h3] = contour(wQ,Ro,GAMMA,[0.1 0.2 0.5 1 2 5 10 20 50 100 200 400]);
% add scaling for wQ* (no outer-wind solution for wQ>wQ*)
plot(16*sqrt(Ro)/27,Ro,':','Color',[0.25 0.25 0.25],'LineWidth',2) 
% add representative real storms from CLE15
scatter(CLE_wQ, CLE_Ro, 16, CLE_colors, 'filled')
set(gca,'Fontsize',14)
xlabel('$\tilde{w_Q}$ = $w_r/(c_D V_m)$','Interpreter','latex')
ylabel('Ro = $V_m/(f r_m)$','Interpreter','latex')
set(gca,'Xscale','log')
set(gca,'Yscale','log')
box on;
clabel(C3,h3)
title('c) $\gamma$ against $\tilde{w_Q}$ and Ro','Fontweight','normal','Interpreter','latex')

gcfsavepdf('/Users/timothycronin/Dropbox/ConfsTalksPubs/Cronin2023_TC-OuterWindModel/Fig03_rar0_vs_RowQ.pdf')

%% save lookup table

lu_table.Ro = Ro;
lu_table.wQ = wQ;
lu_table.r0 = r0;
lu_table.ra = ra;

NRo = num2str(length(Ro));
NwQ = num2str(length(wQ));

%save('/Users/timothycronin/Dropbox/ConfsTalksPubs/Cronin2023_TC-OuterWindModel/lu_table30x40.mat', 'lu_table');
save(['/Users/timothycronin/Dropbox/ConfsTalksPubs/Cronin2023_TC-OuterWindModel/lu_table_' NRo 'x' NwQ '.mat'], 'lu_table');


