function [rr,VV,r0,rmerge] = TC23_windprofile(Vmax,rmax,fcor,C_d,w_cool)
%% TC23_windprofile: code to calculate merged hurricane wind profiles 
% described in Cronin, 2023, GRL - "An analytic model for
% Tropical cyclone outer winds"
% 
% Merged wind profiles are defined as in Chavas, Lin, Emanuel, 2015 -- 
% but computed using analytic solution to outer wind model of Emanuel, 2004
%
% Inputs:
% variable      dimension   units       description
% --------------------------------------------------
%   Vmax        Mx1         [m/s]       maximum wind speed
%   rmax        Mx1         [m]         radius of Vmax
%   fcor        Mx1         [1/s]       Coriolis parameter
%   C_d         Mx1         [-]         drag coefficient in outer region
%   w_cool      Mx1         [m/s]       radiative-subsidence rate
% 
% *** all inputs may be vectors of the same length M ***
%       [NOTE: CkCd [-] of Chavas, Lin, Emanuel - assumed = 1 here]
%
% Outputs:
% variable      dimension   units       description
% ----------------------------------------------------------------
%   rr          MxK         [m]         array of radii for each of M storms
%   VV          MxK         [m/s]       array of wind speeds at rr
%   r0          Mx1         [m]         outer radius where V=0
%   rmerge      Mx1         [m]         radii of merge points between inner and outer solutions
%   Vmerge      Mx1         [m/s]       wind speed at rmerge for each storm
%   rrfracr0    MxK         [-]         array of r/r0 for each storm
%   MMfracM0    MxK         [-]         array of M/M0 at rrfracr0 for each storm
%   rmaxr0      Mx1         [-]         rmax/r0 for each storm
%   MmM0        Mx1         [-]         M/M0 at rmaxr0
%   rmerger0    Mx1         [-]         rmerge/r0
%   MmergeM0    Mx1         [-]         M/M0 at rmerger0
%
% *** some output variables have K points in radius -- 
%     this radial resolution is specified immediately below. 
%     It is fixed in the code and the same for all storms, but can be easily modified ***
%
% Author: Tim Cronin, MIT; twcronin@mit.edu
%
% Functions used: 
%   TC23_windprofile
%   E04_AM_ratio
%   gcfsavepdf
%
% alternative function with more outputs matching those of Chavas, Lin,
% Emanuel, 2015 -- uncomment these outputs towards end of code if desired
%function [rr,VV,r0,rmerge,rrfracr0,MMfracM0,rmaxr0,MmM0,rmerger0,MmergeM0] = ...
%    TC23_windprofile(Vmax,rmax,fcor,C_d,w_cool) 
%

%------------- BEGIN CODE --------------

K = 2000; % number of points to use for radial profile
r_r0 = (1:1:K)/K;
if ~isrow(r_r0)
    r_r0=r_r0';
end

%% calculate merged wind profiles using lookup method
% nondimensional inputs are Ro = Vmax/(fcor*rmax) and wQ = w_cool/(C_d*Vmax)
M = length(Vmax);
Ro = Vmax./(fcor.*rmax);
wQ = w_cool./(C_d.*Vmax);
if ~iscolumn(Ro)
    Ro = Ro';
end
if ~iscolumn(wQ)
    wQ = wQ';
end

load('lu_table_90x120.mat', 'lu_table');

% check that all input variables are within [Ro,wQ] bounds of lookup table
check_bounds = int8(Ro>max(lu_table.Ro))+...
               int8(Ro<min(lu_table.Ro))+...
               int8(wQ>max(lu_table.wQ))+...
               int8(wQ<min(lu_table.wQ));
out_of_bounds = [];
if sum(check_bounds)>0
    out_of_bounds = find(check_bounds>0);
    for i=1:length(out_of_bounds)
        fprintf(1,'note: Ro or wQ out of table bounds for storm %d, output not meaningful \n', out_of_bounds(i))
    end
end
    

% Interpolate to get r0 and rmerge from lookup table
r0          = interp2(log(lu_table.Ro'), log(lu_table.wQ), lu_table.r0', log(Ro), log(wQ));
rmerge      = interp2(log(lu_table.Ro'), log(lu_table.wQ), lu_table.ra', log(Ro), log(wQ));

% M-by-K matrices of Ro, rmerge, r0, and nondim radius for wind calculations
Ro_MxK      = repmat(Ro,1,K);
rmerge_MxK  = repmat(rmerge,1,K);
r0_MxK      = repmat(r0,1,K);
rr          = repmat(r_r0,M,1).*r0_MxK; % array of nondimensional radii 
% (rr is nondimensionalized by each rm, will multiply by each rm on exit from code to make dimensional)

% array of inner wind profiles (dimensionless) following ER11
% Iteration is done to define a slightly modified rm_in, vm_in, and
% Ro_in, which guarantee that v is maximal at r=rm. Modified Ro_in is only
% used for the inner wind profile
nit_vin = 3;
Ro_in_MxK = Ro_MxK; % inner wind profile Ro' will not exactly equal prescribed Ro = vm/(f rm)
for k=1:nit_vin
    rfac = sqrt(sqrt(4.*Ro_in_MxK.^2 + 12.*Ro_in_MxK + 5) - 2.*Ro_in_MxK - 2); % factor relating rm and rm'
    vfac = 2.*Ro_in_MxK.*(1+rfac.^2)./(rfac.*((4*Ro_in_MxK+1)-rfac.^2)); % factor relating vm and vm'
    Ro_in_MxK = Ro_MxK.*vfac.*rfac; % tweak the value of Ro' and repeat a few times
end
v_in = vfac.*rfac.*rr./(2.*Ro_in_MxK.*(1+(rfac.*rr).^2)).*((4*Ro_in_MxK+1)-(rfac.*rr).^2);

% make calculations for outer wind model
gamma = r0./(Ro.*wQ); % \gamma parameter in outer wind model (for M storms)
nterms = max(floor(sqrt(gamma)+40)); % max. needed number of terms in outer wind profile calculation = N
G_out = E04_AM_ratio_matrix(r_r0,gamma,nterms); % matrix calculation of V/V_AMC in outer circulation for M storms

v_out = (r0_MxK.^2-rr.^2)./(2.*Ro_MxK.*rr).*G_out'; % outer wind profile for M storms

% merged wind profile array for all storms: v_in for r<=r_merge; v_out for r>r_merge
VV = v_in.*(rr<=rmerge_MxK) + v_out.*(rr>rmerge_MxK);

%% OUTPUT variables
% make all variables dimensional
rr          = rr.*repmat(rmax,1,K);
VV          = VV.*repmat(Vmax,1,K);
r0          = r0.*rmax;
rmerge      = rmerge.*rmax;
% Vmerge      = zeros(size(r0));
% 
% for i=1:M
%     if sum(out_of_bounds==i)==0 
%         kmerge          = sum(rr(i,:)<rmerge(i));
%         Vmerge(i)       = VV(i,kmerge)+ (VV(i,kmerge+1)-VV(i,kmerge))*(rmerge(i)-rr(i,kmerge))./(rr(i,kmerge+1)-rr(i,kmerge));
%     end
% end
% 
% rrfracr0    = rr./repmat(r0,1,K);
% MMfracM0    = (rr.*VV + 0.5.*repmat(fcor,1,K).*rr.^2)./...
%               (0.5.*repmat(fcor,1,K).*repmat(r0,1,K).^2);
% rmaxr0      = rmax./r0;
% MmM0        = (rmax.*Vmax + 0.5*fcor.*rmax.^2)/(0.5*fcor.*r0.^2);
% rmerger0    = rmerge./r0;
% MmergeM0    = (rmerge.*Vmerge + 0.5*fcor.*rmerge.^2)./(0.5*fcor.*r0.^2);

end

function [G] = E04_AM_ratio_matrix(r,L,series_length)
%E04_AM_ratio - power series solution for outer wind profile 
% as function of radius - this function computes the ratio of the
% tangential wind to the wind that would be obtained from angular momentum
% conservation of inflow from rest at radius r_0
% 

% input arguments:
% r = nondimensional array of r/r_0 - K elements in length
% L = gamma = r_0/(w_Q Ro) [dimensionless]
%   = (r_0/r_m)/[-w_rad/(c_D v_m) (v_m/f r_m)] 
%   = (f r_0 c_D)/(-w_rad) - key solution control parameter, typically >>1, equal to r_0/w_Q from Emanuel 2004 
% L can be an array of multiple different values of \Lambda - M elements in length
% series_length = N = number of terms to calculate in power series 

% output arguments:
% G = V/V_AMC = ratio of tangential wind speed to wind speed of
% angular-momentum-conserving profile

% solution method is to construct two NxN matrices of the coefficients that
% left-multiply powers of L (a NxM matrix) to give the numerator and 
% denominator power series coefficients (N terms for each of the K elements of L), 
% then a matrix of powers of x=(1-r/r0) (size KxN) left-multiplies this
% result

N = series_length;
K = length(r);
if ~iscolumn(r)
    r = r';
end
M = length(L);
if ~isrow(L)
    L = L';
end

% coefficients in series for denominator and numerator, respectively -
% initialize with identity matrix
PP_den = eye(N+1,N+1);
PP_num = eye(N,N);

% construct matrix of powers of (1-r) (nth column is (1-r)^(n-1)
XX = ones(K,N);
for i=2:N
    XX(:,i) = XX(:,i-1).*(1-r);
end

% construct matrix of powers of L (nth row is L.^(n-1))
LL = ones(N,M);
for i=2:N
    LL(i,:) = LL(i-1,:).*L;
end

% construct coefficient matrix (NxN), nth row is for x.^(n-1) and mth column is for L.^(n-1)
% diagonal follows PP(i,i) = 1./((i-1)!)^2, or recurrence that each diagonal element is only related to the previous one
% PP(4,3) has value -1/36
% PP(i>1,1) and PP(i>2,2) all = 0
% PP(:,3) for i>4 is equal to PP(i-1,3)*(i-2)/(2*(i-1)) -- special case of 
% general lower-diagonal entries with j>=3 and i>j which has recurrence given by:
% PP(i,j) = PP(i-1,j-1)/(i-1)^2 + PP(i-1,j)*(i-2)/(2*(i-1)) - PP(i-2,j-1)/(i-1)^2  
PP_den(4,3) = -1/36;
for j=3:N+1 % loop over columns (index j)
    for i=3:N+1 % loop over rows (index i)
        if j==i
            PP_den(i,j) = PP_den(i-1,j-1)/(i-1)^2;
        elseif (j>=3)&&(i>j)&&(i>4)
            PP_den(i,j) = PP_den(i-1,j-1)/(i-1)^2 + PP_den(i-1,j)*(i-2)/(2*(i-1)) - PP_den(i-2,j-1)/(i-1)^2;
        end
    end
end
for i=2:N % fill numerator matrix
    PP_num(i,:) = i*PP_den(i+1,2:N+1);
end

num = XX*(PP_num*LL);
den = XX*(PP_den(1:N,1:N)*LL); % truncate denominator coefficients to NxN

G = num./den;

end

